//
// dist.DObject - the foundation for all information sharing in the
//                distributed environment
//
// This code is Copyright (C) 1997, go2net Inc. Permission is granted for
// any use so long as this header remains intact.
//
// Originally published in Deep Magic:
//     <URL:http://www.go2net.com/internet/deep/>
//
// The code herein is provided to you as is, without any warranty of any
// kind, including express or implied warranties, the warranties of
// merchantability and fitness for a particular purpose, and
// non-infringement of proprietary rights.  The risk of using this code
// remains with you.

package dist;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

public class DObject extends Hashtable
{
    //
    // DObject public data members

    public String oid;

    //
    // DObject public constructors

    // objects should never be constructed. instead, they should be
    // requested from the object manager
    public DObject (String oid, DObjectManager manager)
    {
        this.oid = oid;
        _manager = manager;
    }

    //
    // DObject public member functions

    public int getValue (String name, int defval)
    {
        Integer val = (Integer)get(name);
        return (val == null) ? defval : val.intValue();
    }

    public String getValue (String name, String defval)
    {
        String val = (String)get(name);
        return (val == null) ? defval : val;
    }

    public int[] getValue (String name, int[] defval)
    {
        int[] val = (int[])get(name);
        return (val == null) ? defval : val;
    }

    public String[] getValue (String name, String[] defval)
    {
        String[] val = (String[])get(name);
        return (val == null) ? defval : val;
    }

    public void setValue (String name, int value) throws IOException
    {
        _manager.attributeChanged(this, name, new Integer(value));
    }

    public void setValue (String name, String value) throws IOException
    {
        _manager.attributeChanged(this, name, value);
    }

    public void setValue (String name, int[] value) throws IOException
    {
        _manager.attributeChanged(this, name, value);
    }

    public void setValue (String name, String[] value) throws IOException
    {
        _manager.attributeChanged(this, name, value);
    }

    public void addSubscriber (Subscriber s)
    {
        _subscribers.addElement(s);
    }

    public void handleEvent (Event evt) throws IOException
    {
        // change attribute values for attribute value changes
        if (evt.type == Event.ATTR_CHANGED) {
            if (evt.value != null) put(evt.name, evt.value);
            else remove(evt.name);
        }

        notifySubscribers(evt);
    }

    public synchronized void notifySubscribers (Event evt) throws IOException
    {
        for (int i = 0; i < _subscribers.size(); i++) {
            Subscriber s = (Subscriber)_subscribers.elementAt(i);

            if (s == null) {
                System.err.println("Ack! Null subscriber of " + oid + ": " + i);
                _subscribers.removeElementAt(i--);
                continue;
            }

            if (!s.handleEvent(this, evt)) _subscribers.removeElementAt(i--);
        }
    }

    public void readFrom (DataInputStream in) throws IOException
    {
        for (int elements = in.readInt(); elements > 0; elements--) {
            put(in.readUTF(), Transport.readValue(in));
        }
    }

    public void writeTo (DataOutputStream out) throws IOException
    {
        out.writeInt(size());

        Enumeration vals = keys();
        while (vals.hasMoreElements()) {
            String key = (String)vals.nextElement();
            out.writeUTF(key);
            Transport.writeValue(out, get(key));
        }
    }

    //
    // DObject protected data members

    protected DObjectManager _manager;
    protected Vector _subscribers = new Vector();
}
