//
// dist.Event - communicates object existence and value changes between
//              sharers of information
//
// This code is Copyright (C) 1997, go2net Inc. Permission is granted for
// any use so long as this header remains intact.
//
// Originally published in Deep Magic:
//     <URL:http://www.go2net.com/internet/deep/>
//
// The code herein is provided to you as is, without any warranty of any
// kind, including express or implied warranties, the warranties of
// merchantability and fitness for a particular purpose, and
// non-infringement of proprietary rights.  The risk of using this code
// remains with you.

package dist;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

public class Event
{
    //
    // Event public constants

    public final static byte OBJECT_CREATED = 0;
    public final static byte OBJECT_DELETED = 1;
    public final static byte ATTR_CHANGED = 2;

    public final static String[] typeTrans = {
        "OBJECT_CREATED",
        "OBJECT_DELETED",
        "ATTR_CHANGED",
    };

    //
    // Event public data members

    public byte type;
    public String oid;

    public String name;
    public Object value;

    //
    // Event public constructor

    public Event (byte type, String oid)
    {
        this.type = type;
        this.oid = oid;
    }

    public Event (DataInputStream din) throws IOException
    {
        type = din.readByte();
        oid = din.readUTF();

        if (type == ATTR_CHANGED) {
            name = din.readUTF();
            value = Transport.readValue(din);
        }
    }

    //
    // Event public member functions

    public void writeTo (DataOutputStream dout) throws IOException
    {
        dout.writeByte(type);
        dout.writeUTF(oid);

        if (type == ATTR_CHANGED) {
            dout.writeUTF(name);
            Transport.writeValue(dout, value);
        }
    }

    public String toString ()
    {
        String str = typeTrans[type] + " " + oid;
        if (type == ATTR_CHANGED) {
            str = str + " (" + name + "=" + value + ")";
        }
        return str;
    }
}
